/****************************************************************************
*
* Atmel Corporation
*
* File              : main.c
* Compiler          : IAR EWAAVR 4.20A
* Revision          : $Revision: 0.99 $
* Date              : $Date: Thursday, September 20, 2006 $
* Updated by        : $Author: Jens Rotter $
*
* Support mail      : avr@atmel.com
*
* Supported devices : ATmega88
*
* AppNote           : ?
* 
* Description       : Touchscreen Controller 4wire (Lowlevel/Rtouch Measurement) and 5wire (Lowlevel Measurement)
*                     
*
*
****************************************************************************/

/****************************************************************************
  Libraries
****************************************************************************/
#include "Touchscreen644.h"

#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include <util/delay.h>  
#include <avr/sleep.h>
#include <stdio.h>
#include <stdlib.h>
#include "uart.h"
// UART file descriptor
// putchar and getchar are in uart.c
FILE uart_str = FDEV_SETUP_STREAM(uart_putchar, uart_getchar, _FDEV_SETUP_RW);
       


/****************************************************************************
  Global definitions
****************************************************************************/
// Set Interface
#define UART_INTERFACE

// Flags to interpret
#define FLAG_REGISTER_MASK 255

// Size of Buffer for Converting unsigned int Value to ASCII
#define STRING_BUFFER_SIZE 5

#define READY_TO_TX 1
#define NOT_READY_TO_TX 0
#define READY_TO_RX 1
#define NOT_READY_TO_RX 0
volatile unsigned char TX_STATUS = NOT_READY_TO_TX;
volatile unsigned char RX_STATUS = NOT_READY_TO_RX;
/****************************************************************************
  Function definitions
****************************************************************************/
unsigned char Recognize_Event(void);
void Send_over_UART(void);

/****************************************************************************
  Global variable
****************************************************************************/
// Touchscreen Interface
extern Interface Touchscreen_Data;
Interface Buffer_Touchscreen_Data;
Interface Pre_Buffer_Touchscreen_Data;
/****************************************************************************
  main loop
****************************************************************************/
ISR( PCINT3_vect )
{

	//if video code ready to receive
	if( TestBit( PIND, 1 ) )
	{
		if( TX_STATUS )
		{
				PORTB = (char)(Buffer_Touchscreen_Data.x_pos );
				PORTC = (char)(Buffer_Touchscreen_Data.y_pos );
		}

	}


}
int main( void )
{  
        // USART Initialization
          //init the UART -- uart_init() is in uart.c
		  uart_init();
		  stdout = stdin = stderr = &uart_str;
		  fprintf(stdout,"Starting timers...\n\r");
      	//pin configuaration for communication
		DDRB = 0xff;
		DDRC = 0xff;
		PORTB = 0;
		PORTC = 0;

		//port c is an input and output
		DDRD = DDRA | 0b00000001;//pin D0 for TX, D1 for RX
		PORTD = PORTD | 0x00000010; //turn on pull up resistors for the input pin


        // Touchscreen Initialization
        Touchscreen_Init();

		 // Set Pin Change Interrupt Source
        SetBit(PCMSK3, PCINT25);
        // Enable Pin Change Interrupt
        SetBit(PIN_CHANGE_INTERRUPT_CONTROL_REGISTER, PCIE3); 


        // Enable global Interrupts
        sei();

        // Infinite Loop
	for (;;)
	{       
                if(Recognize_Event())
                {   
					//RX_STATUS = PINA >> 6;
                    #if defined UART_INTERFACE       
                          Send_over_UART();
                    #endif
					
                }
	}
} 

unsigned char Recognize_Event(void)
{
        cli();
        Pre_Buffer_Touchscreen_Data = Buffer_Touchscreen_Data;
        // Read Data
        Buffer_Touchscreen_Data = Touchscreen_Data;
        // Mask Flag_Register
        Buffer_Touchscreen_Data.Flag_Register &= FLAG_REGISTER_MASK;
        // Clear Joblist
        Touchscreen_Data.Flag_Register ^= Buffer_Touchscreen_Data.Flag_Register;
       
        sei();
        return Buffer_Touchscreen_Data.Flag_Register;
}

void Send_over_UART(void)
{

        if(TestBit(Buffer_Touchscreen_Data.Flag_Register, FLAG_REGISTER_OVERFLOW))
        {
            // Overflow: X/Y-Coordinates were overwritten
            fprintf(stdin, "Overflow ");
			TX_STATUS = NOT_READY_TO_TX;
        }
        else
        {
              if(TestBit(Buffer_Touchscreen_Data.Flag_Register, FLAG_REGISTER_START))
              {
                  // Start of Programm --> e.g. collect Calibration Points for higher Level Software
                  fprintf(stdin, "Start... ");
              }
              
              if(TestBit(Buffer_Touchscreen_Data.Flag_Register, FLAG_REGISTER_SLEEP))
              {
                  // Nothing to do... --> e.g. Powerdown
                  fprintf(stdin, "Sleep ");
					TX_STATUS = NOT_READY_TO_TX;
                  // Power down
                  //SMCR = (2<<SM0)|(1<<SE);
                  //__sleep(); 
              }
                    
              if(TestBit(Buffer_Touchscreen_Data.Flag_Register, FLAG_REGISTER_TOUCH))
              {
                  // Touch detected
                  fprintf(stdin, "Touch ");
				  TX_STATUS = READY_TO_TX;
              }
              
              if(TestBit(Buffer_Touchscreen_Data.Flag_Register, FLAG_REGISTER_END))
              {
                  // End of Touch detected
                  fprintf(stdin, "End ");
				  TX_STATUS = NOT_READY_TO_TX;
              }
             
              if(TestBit(Buffer_Touchscreen_Data.Flag_Register, FLAG_REGISTER_COORDINATES))
              {
                  // X/Y-Coordinates ready
				  fprintf( stdin, "(%d, %d)", Buffer_Touchscreen_Data.x_pos, Buffer_Touchscreen_Data.y_pos);
					TX_STATUS = READY_TO_TX;
				   //RX_STATUS = 1;
				
              }                          
        }
		PORTD = PORTD | (TX_STATUS<<0); //set the output pin D0 to the tx_status
		//transmit the X position
				
}
